function [layout chassis_type echo_out]=CBMX_query_layout(ser_obj)
% D. Stahl 22.2.13
% queries Layout of system
% response is nx4 matrix. 3rd column is number of lasers in according slot;
% 4th column is laser type: 0: TLS (fully tunable laser), 
%       1: TLC (ITU tunable laser), 2: TLW (Wide line width Laser)

verbose=0;
timeout=2;
if ser_obj.BytesAvailable
    fread(ser_obj,ser_obj.BytesAvailable); %clear receive buffer
end
fprintf(ser_obj,'lay?;');
tic;
while ser_obj.BytesAvailable==0 && toc<timeout
    pause(0.1);
end
pause(0.05);
if ser_obj.BytesAvailable
    data=char(fread(ser_obj,ser_obj.BytesAvailable))';
    raw_res=regexp(data,'\n','split'); %read buffer and break it into lines
    idx=1;
    for i=1:length(raw_res)
        if ~isempty(strfind(raw_res{i},'ERR'))
            error(raw_res{i});
            layout=[];
        else
            if isempty(strfind(raw_res{i},'lay?')) %check if echo
                if ~isempty(strfind(raw_res{i},'SYSTEM')) % line contains chassis type
                    chassis_type=raw_res{i}(8:end);
                else
                    if ~isempty(strfind(raw_res{i},'EMP')) % check if empty slot
                        layout(idx,1:2)=sscanf(raw_res{i},'%d,%d');
                        layout(idx,3)=0;
                    else
                        
                        if ~isempty(strfind(raw_res{i},'TLS')) % check if TLS laser
                            layout(idx,1:3)=sscanf(raw_res{i},'%d,%d,TLS%d')';
                            layout(idx,4)=0;
                        elseif ~isempty(strfind(raw_res{i},'TLC')) % check if TLC laser
                            layout(idx,1:3)=sscanf(raw_res{i},'%d,%d,TLC%d')';
                            layout(idx,4)=1;
                        else %else TLW laser
                            layout(idx,1:3)=sscanf(raw_res{i},'%d,%d,TLW%d')';
                            layout(idx,4)=2;
                        end    
                    end
                    idx=idx+1;
                end
            else
                echo_out=raw_res{i};
            end
        end
    end
    if ~exist('echo_out'), echo_out=[]; end
    if verbose
        disp(strcat('Chassis: ',chassis,','));
        for idx=1:(i-1)
            disp(sprintf('Slot %d,%d :%d tunable lasers',layout(idx,:)));
        end
    end
else
    error('no data received');
end
