% This file demonstrates the usage of the Matlab functions to control
% CoBrite tunable Laser Chassis
% D. Stahl 20.2.13

delete(instrfind); %first clean up before doing anything

% =================================================================
%Parameter
% =================================================================
com_port='com5'; % COM Port setting to operate via USB/serial Port,
%look up serial port number in Device Manaager of Windows
IP_address='192.168.0.1'; % IP address of Laser chassis
% use the following line to access the chassis via USB/serial emulation and
% comment out the tcpip connection
 ser_obj=serial(com_port,'BaudRate',115200);
% use the following line to access chassis via Ethernet/Telnet and comment
% out the command for serial connection
% ser_obj = tcpip(IP_address, 10001); % open telnet connection via Ethernet
max_time=45; %max time to wait after tuning [s]
ptime=2; % pause time used after setting a value

% enter here ports to test
Port= ...
    [1 2 1]; % do not use wildcards here
% note: Echo needs to be off to make code function

% =================================================================
% Code
% =================================================================
warnings={};
clear port;
wi=1;
fopen(ser_obj); % open serial port
fprintf(ser_obj,';'); % flush of Laser chassis buffer by sending command termination signal
pause(0.1);
while ser_obj.BytesAvailable % flush Rx buffer
    temp=fscanf(ser_obj,'%c',ser_obj.BytesAvailable);
end
if 1 %system commands
    fprintf('Testing system level commands\n');
    if 1 %interlock query
        interlock=CBMX_query_interlock_state(ser_obj);
        fprintf('interlock state : %d\n',interlock);
    end
    if 1 %echo
        echo_out=CBMX_query_echo(ser_obj);
        ok=CBMX_set_echo(ser_obj,~echo_out); % set opposite of current state
        stat=CBMX_query_echo(ser_obj);
        if stat==~echo_out
            fprintf('echo successfully set to %d\n',~echo_out);
        else
            warnings{wi}=sprintf('Port %d-%d-%d: Warning! Get/set echo not working!',Port(ii,:));
            disp(warnings{wi});
            wi=wi+1;
        end
        CBMX_set_echo(ser_obj,1); %switch echo on
    end
    
    if 1 %layout
        layout=CBMX_query_layout(ser_obj);
        fprintf('Laser chassis layout\n');
        for i=1:size(layout,1)
            fprintf('Slot %d-%d, Laser Type: %d, # of Lasers: %d\n',layout(i,:));
        end
        fprintf('\nLaser Type definition:  0= TLS, 1= TLC, 2= TLW\n');
    end
    
    if 1 %system controller info
        [FW_version HW_version controller_type serial_number]=CBMX_query_card_info(ser_obj);
        fprintf('FW Version: %d.%d.%d, HW Version: %d, controller_type: %s, Ser No: %s\n',...
            FW_version,HW_version,controller_type,serial_number);
    end
end

fprintf('=================================================\n=================================================\n');
fprintf('testing Laser Ports\n');
for ii=1:size(Port,1)
    port(ii).address=Port(ii,:);
    fprintf('=================================================\n');
    fprintf('Testing port: %d-%d-%d\n',Port(ii,:));
    LaserTypeTested = layout((Port(ii,1)-1)*12+Port(ii,2),4);
    
    if 1 %laser controller info
        [port(ii).FW_version port(ii).HW_version port(ii).card_type port(ii).serial_number]=CBMX_query_card_info(ser_obj,Port(ii,1:2));
        fprintf('FW Version: %d.%d.%d, HW Version: %d.%d.%d, card type: %s, Ser No: %s\n',...
            port(ii).FW_version,port(ii).HW_version,port(ii).card_type,port(ii).serial_number);
    end
    
    if 1 %Port commands
        if 1 % Port state
            disp('testing Output state');
            port(ii).stat.target=~CBMX_query_port_state(ser_obj,Port(ii,:)); %chose opposite of current state
            CBMX_set_port_state(ser_obj,Port(ii,:),port(ii).stat.target);
            pause(0.5);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            port(ii).stat.read=CBMX_query_port_state(ser_obj,Port(ii,:));
            fprintf('Output status setting/reading ok to %d\n',port(ii).stat.target);
            if port(ii).stat.target~=port(ii).stat.read
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Output status setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
        end
        
        disp('Enabling Laser again...');
        CBMX_set_port_state(ser_obj,Port(ii,:),1); %always switch Port on for next steps
        wait_tuning(ser_obj,Port(ii,:),max_time);
        if 1 % Power
            disp('testing Power command');
            port(ii).power.lim=CBMX_query_port_power_limit(ser_obj,Port(ii,:));
            port(ii).power.target=gen_random_number(port(ii).power.lim);
            CBMX_set_port_power(ser_obj,Port(ii,:),port(ii).power.target);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(12); % leave in for power tuning
            port(ii).power.read=CBMX_query_port_power(ser_obj,Port(ii,:));
            fprintf('Power setting/reading ok, Limits: %2.2fdBm to %2.2fdBm, target: %2.2fdBm, actual: %2.2fdBm\n',...
                port(ii).power.lim,port(ii).power.target,port(ii).power.read);
            if abs( port(ii).power.target-port(ii).power.read)>0.07
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Power setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
        end
        
        if 1 % FTF
            disp('testing FTF command');
            port(ii).FTF.lim=CBMX_query_port_FTF_limit(ser_obj,Port(ii,:));
            port(ii).FTF.target=gen_random_number(port(ii).FTF.lim);
            CBMX_set_port_FTF(ser_obj,Port(ii,:),port(ii).FTF.target);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            port(ii).FTF.read=CBMX_query_port_FTF(ser_obj,Port(ii,:));
            fprintf('FTF setting/reading ok, Limits: %2.3fGHz to %2.3fGHz, target: %2.3fGHz, actual: %2.3fGHz\n',...
                port(ii).FTF.lim,port(ii).FTF.target,port(ii).FTF.read);
            if abs(port(ii).FTF.target-port(ii).FTF.read)>0.002
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! FTF setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
        end
        
        if 1 % Frequency/WL setting
            disp('testing Frequency command');
            port(ii).frequency.lim=CBMX_query_port_frequ_limit(ser_obj,Port(ii,:));
            port(ii).frequency.target=gen_random_number(port(ii).frequency.lim);
            if LaserTypeTested, port(ii).frequency.target = round(port(ii).frequency.target*20)/20; end % round to nearest ITU channel if TLC is used
            CBMX_set_port_frequ(ser_obj,Port(ii,:),port(ii).frequency.target);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            port(ii).frequency.read=CBMX_query_port_frequ(ser_obj,Port(ii,:));
            fprintf('Frequency setting/reading ok, Limits: %3.4fTHz to %3.4fTHz, target: %3.4fTHz, actual: %3.4fTHz\n',...
                port(ii).frequency.lim,port(ii).frequency.target,port(ii).frequency.read);
            if abs(port(ii).frequency.target-port(ii).frequency.read)>0.0002
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Frequency setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
        end
        
        if 1 && ~LaserTypeTested % Wavelength setting,test only if fully tunable laser (TLS)
            disp('testing wavelength command');
            port(ii).wav.lim=CBMX_query_port_wav_limit(ser_obj,Port(ii,:));
            port(ii).wav.target=gen_random_number(port(ii).wav.lim);
            CBMX_set_port_wavelength(ser_obj,Port(ii,:),port(ii).wav.target);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            port(ii).wav.read=CBMX_query_port_wav(ser_obj,Port(ii,:));
            fprintf('Wavelength setting/reading ok, Limits: %4.3fnm to %4.3fnm, target: %4.3fnm, actual: %4.3fnm\n',...
                port(ii).wav.lim,port(ii).wav.target,port(ii).wav.read);
            if abs(port(ii).wav.target-port(ii).wav.read)>0.002
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Wavelength setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
        end
        
        if 1 % configuration of port using conf command
            disp('testing config command, change all parameter at the same time');
            port(ii).conf.lim=CBMX_query_limits(ser_obj,Port(ii,:)); %note limits can be queried only for one port at a time
            port(ii).conf.target=[gen_random_number(port(ii).conf.lim(1:2)) gen_random_number([-port(ii).conf.lim(3) port(ii).conf.lim(3)])...
                gen_random_number(port(ii).conf.lim(4:5)) 1 -1] ;
            if LaserTypeTested % if ITU tunable laser round to nearest 50GHz ITU channel
                port(ii).conf.target(1) = round(port(ii).conf.target(1)*20)/20;
                port(ii).conf.target(2) = 0; %FTF tuning not allowed if new ITU channel is set
            end % round to nearest ITU channel and use no offset if TLC or TWL is used
            CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),port(ii).conf.target(4),port(ii).conf.target(5));
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(ptime);
            port(ii).conf.read=CBMX_query_port_config(ser_obj,Port(ii,:));
            if abs(port(ii).conf.target(1)-port(ii).conf.read(1))<=0.0002 && ... % Frequency
                    abs(port(ii).conf.target(2)-port(ii).conf.read(2))<=0.001 && ... % FTF
                    abs(port(ii).conf.target(3)-port(ii).conf.read(3))<=0.1    && ... % Power
                    port(ii).conf.target(4)==port(ii).conf.read(4)
                disp('Port Conf setting/reading all parameters ok');
            else
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Config setting/reading, all parameters at the same time not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
            fprintf('Frequency; Limits: %2.4fTHz to %2.4fTHz, target: %2.4fTHz, actual: %2.4fTHz\n',...
                port(ii).conf.lim(1:2),port(ii).conf.target(1),port(ii).conf.read(1));
            fprintf('FTF; Limits: %2.3fGHz to %2.3fGHz, target: %2.3fGHz, actual: %2.3fGHz\n',...
                [-port(ii).conf.lim(3) port(ii).conf.lim(3)],port(ii).conf.target(2),port(ii).conf.read(2));
            fprintf('Power; Limits: %2.2fdBm to %2.2fdBm, target: %2.2fdBm, actual: %2.2fdBm\n',...
                port(ii).conf.lim(4:5),port(ii).conf.target(3),port(ii).conf.read(3));
            fprintf('Output state : %d, Tuning state : %d, Dither state: %d\n',port(ii).conf.read(4:end));
            
            disp('testing config command, change frequency');
            port(ii).conf.target(1)=gen_random_number(port(ii).conf.lim(1:2));
            if LaserTypeTested % round to nearest ITU channel if TLC laser and set FTF to 0
                port(ii).conf.target(1) = round(port(ii).conf.target(1)*20)/20;
                port(ii).conf.target(2) = 0; %FTF tuning not allowed if new ITU channel is set
            end
            CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),port(ii).conf.target(4),port(ii).conf.target(5));
            pause(ptime);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(ptime);
            
            disp('testing config command, change FTF offset');
            port(ii).conf.target(2)=gen_random_number([-port(ii).conf.lim(3) port(ii).conf.lim(3)]);
            CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),port(ii).conf.target(4),port(ii).conf.target(5));
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(ptime);
            
            if ~(LaserTypeTested==1) % power not settable for TLC laser
                disp('testing config command, change power');
                port(ii).conf.target(3)=gen_random_number(port(ii).conf.lim(4:5));
                CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),port(ii).conf.target(4),port(ii).conf.target(5));
                wait_tuning(ser_obj,Port(ii,:),max_time);
                pause(3*ptime);
            end
            
            port(ii).conf.read=CBMX_query_port_config(ser_obj,Port(ii,:));
            if abs(port(ii).conf.target(1)-port(ii).conf.read(1))<=0.0002 && ... % Frequency
                    abs(port(ii).conf.target(2)-port(ii).conf.read(2))<=0.001 && ... % FTF
                    abs(port(ii).conf.target(3)-port(ii).conf.read(3))<=0.1    && ... % Power
                    port(ii).conf.target(4)==port(ii).conf.read(4)
                disp('Port Conf setting/reading change frequency ok');
            else
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Config setting/reading change frequency not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
            fprintf('Frequency; Limits: %2.4fTHz to %2.4fTHz, target: %2.4fTHz, actual: %2.4fTHz\n',...
                port(ii).conf.lim(1:2),port(ii).conf.target(1),port(ii).conf.read(1));
            fprintf('FTF; Limits: %2.3fGHz to %2.3fGHz, target: %2.3fGHz, actual: %2.3fGHz\n',...
                [-port(ii).conf.lim(3) port(ii).conf.lim(3)],port(ii).conf.target(2),port(ii).conf.read(2));
            fprintf('Power; Limits: %2.2fdBm to %2.2fdBm, target: %2.2fdBm, actual: %2.2fdBm\n',...
                port(ii).conf.lim(4:5),port(ii).conf.target(3),port(ii).conf.read(3));
            fprintf('Output state : %d, Tuning state : %d, Dither state: %d\n',port(ii).conf.read(4:end));
            
            disp('testing config command, switch off');
            CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),0,-1);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(ptime);
            port(ii).conf.read=CBMX_query_port_config(ser_obj,Port(ii,:));
            if port(ii).conf.read(3)<-10 % Power
                disp('Port switch off using Conf setting/reading ok');
            else
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Port switch off using config setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
            
            disp('testing config command, switch on');
            if LaserTypeTested
                port(ii).conf.target(1) = round(port(ii).conf.target(1)*20)/20;
                port(ii).conf.target(2) = 0;
            end % round to nearest ITU channel and use no offset if TLC is used
            CBMX_set_port_config(ser_obj,Port(ii,:),port(ii).conf.target(1),port(ii).conf.target(2),port(ii).conf.target(3),port(ii).conf.target(4),1);
            pause(ptime);
            wait_tuning(ser_obj,Port(ii,:),max_time);
            pause(ptime);
            port(ii).conf.read=CBMX_query_port_config(ser_obj,Port(ii,:));
            if abs(port(ii).conf.target(3)-port(ii).conf.read(3))<=0.1 % Power
                disp('Port switch off using Conf setting/reading ok');
            else
                warnings{wi}=sprintf('Port %d-%d-%d: Warning! Port switch off using config setting/reading not working',Port(ii,:));
                disp(warnings{wi});
                wi=wi+1;
            end
            
        end
        
        if 1 % monitor
            disp('testing monitor values');
            port(ii).wav.mon=CBMX_query_monitor_values(ser_obj,Port(ii,:));
            fprintf('LaserDiode chip temp: %2.2fC, LaserDiode base temp %2.2fC, LaserDiode current %3.1fmA, TEC current %3.1fmA\n',port(ii).wav.mon);
        end
        
        if 1 % dither
            disp('testing dither');
            curr=CBMX_query_port_dither(ser_obj,Port(ii,:));
            if curr==-1
                disp('Dither is not supported for this laser port!');
            else
                port(ii).dither.target=~curr; %chose opposite of current state
                CBMX_set_port_dither(ser_obj,Port(ii,:),port(ii).dither.target);
                pause(0.5);
                wait_tuning(ser_obj,Port(ii,:),max_time);
                pause(ptime);
                port(ii).dither.read=CBMX_query_port_dither(ser_obj,Port(ii,:));
                if port(ii).dither.target~=port(ii).dither.read
                    warnings{wi}=sprintf('Port %d-%d-%d: Warning! Dither status setting/reading not working',Port(ii,:));
                    disp(warnings{wi});
                    wi=wi+1;
                end
                fprintf('Dither status reading %d\n',port(ii).dither.read);
            end
        end
        CBMX_set_port_state(ser_obj,Port(ii,:),0); % switch port off
        pause(0.5);
    end
end
if wi > 1
    disp('Warnings summary:');
    for i=1:wi-1
        disp(warnings{i});
    end
else
    disp('No errors/warnings');
end
fclose(ser_obj);